﻿using Krypton.Toolkit;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Wa.Api;
using WaMarketProApp.Extensions;
using WaMarketProApp.Model;
using WaMarketProApp.Utils;

namespace WaMarketProApp.ChildFrom
{
    public partial class FilterControl : UserControl
    {
        private CancellationTokenSource cancellationToken = new CancellationTokenSource();
        private List<FilterResultModel> list = new List<FilterResultModel>();
        private int fail = 0;
        private int success = 0;
        public override string Text { get; set; } = "Filter";
        public WaApi WaApi { get; set; }
        public FilterControl()
        {
            this.Text = LangUtils.GetLang()?.MenuWaFilter;
            InitializeComponent();
        }

        private void btnImport_Click(object sender, EventArgs e)
        {
            try
            {

                var dialog = new OpenFileDialog
                {
                    Filter = @"Text File|*.txt",
                    Title = @"Select Text File"
                };
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    var path = dialog.FileName;
                    var list = File.ReadAllLines(path)
                        .Select(s => Regex.Replace(s, @"[^0-9]+", ""))
                         .Where(s => !string.IsNullOrEmpty(s)).ToArray();
                    this.txtList.Lines = list;
                    this.labTotal.Text = list.Count().ToString();
                }
            }
            catch (Exception ex)
            {
                KryptonMessageBox.Show(ex.Message, @"Error", MessageBoxButtons.OK, KryptonMessageBoxIcon.Error, showCtrlCopy: false);

            }
        }

        private void btnFilterClear_Click(object sender, EventArgs e)
        {
            this.txtList.Clear();
            this.labTotal.Text = "0";
        }

        private void btnFilterStart_Click(object sender, EventArgs e)
        {

            if (string.IsNullOrEmpty(this.txtList.Text.Trim()))
            {
                KryptonMessageBox.Show("Please import your Phone number!", "Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Error, showCtrlCopy: false);
                return;
            }
            this.labExist.Text = "0";
            this.labNoExist.Text = "0";
            var delay = (int)this.nudDelay.Value;
            this.list = new List<FilterResultModel>();
            var phoneList = this.txtList.Lines.Where(s => !string.IsNullOrEmpty(s)).ToList();
            this.btnFilterStart.Enabled = false;
            this.btnFilterStop.Enabled = true;

            var token = this.cancellationToken.Token;
            Task.Factory.StartNew(() =>
            {
                this.Filter(phoneList, delay, token);

            }, token).ContinueWith(t =>
            {
                this.cancellationToken = new CancellationTokenSource();
                this.InvokeOnUiThreadIfRequired(() =>
                {
                    this.btnFilterStart.Enabled = true;
                    this.btnFilterStop.Enabled = false;
                    KryptonMessageBox.Show("Filter Finish!", "Info",
                          MessageBoxButtons.OK, KryptonMessageBoxIcon.Information, showCtrlCopy: false);
                });
            });
        }


        private void Filter(List<string> phoneList, int delaly, CancellationToken cancellation)
        {
            foreach (var item in phoneList)
            {
                if (cancellation.IsCancellationRequested)
                {
                    break;
                }
                var r = this.WaApi.Validator(item);
                SpinWait.SpinUntil(() => false, delaly * 1000);
                var status = r ? "Exist" : "NoExist";
                this.list.Add(new FilterResultModel { PhoneNumber = item, Status = status });
                this.InvokeOnUiThreadIfRequired(() =>
                {
                    this.dgvList.Rows.Insert(0, new object[] { item, status });
                    this.dgvList.Rows[0].Cells[1].Style.ForeColor = r ? Color.Green : Color.Red;
                    this.dgvList.Rows[0].Cells[1].Style.Font = new Font(FontFamily.GenericSansSerif, 8.0F, FontStyle.Bold);
                    if (r)
                    {
                        success++;
                        this.labExist.Text = success.ToString();
                    }
                    else
                    {
                        fail++;
                        this.labNoExist.Text = fail.ToString();
                    }

                });
            }
        }

        private void btnFilterStop_Click(object sender, EventArgs e)
        {
            this.cancellationToken.Cancel();
        }

        private void btnExport_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.dgvList.Rows.Count == 0)
                {
                    KryptonMessageBox.Show("No Data!", @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Warning, showCtrlCopy: false);
                    return;
                }

                var resultList = list.Where(s => s.Status == "Exist").Select(g => g.PhoneNumber).ToArray();
                var dialog = new SaveFileDialog
                {
                    Filter = @"Text File|*.txt",
                    FileName = $"Exist-PhoneList-{DateTime.Now.ToString("yyyy-MM-ddHHmmss")}"
                };
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    var path = dialog.FileName;
                    File.WriteAllLines(path, resultList, Encoding.UTF8);
                    KryptonMessageBox.Show("Save Success!", @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Information, showCtrlCopy: false);

                }
            }
            catch (Exception ex)
            {

                KryptonMessageBox.Show(ex.Message, @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Error, showCtrlCopy: false);
            }
        }

        private void btnFilterClearR_Click(object sender, EventArgs e)
        {
            this.dgvList.Rows.Clear();
            this.list.Clear();
            this.fail = 0;
            this.success = 0;
            this.labExist.Text = "0";
            this.labNoExist.Text = "0";
        }

        private void FilterControl_Load(object sender, EventArgs e)
        {
            var model = LangUtils.GetLang();
            if (model != null)
            {
                this.Text = model.MenuWaFilter;
                kryptonLabel4.Text = model.LabDelay;
                kryptonLabel1.Text = model.LabTotal;
                kryptonLabel2.Text = model.LabSucces;
                kryptonLabel3.Text = model.LabFail;
                btnImport.Text = model.BtnImport;
                btnFilterClear.Text = model.BtnClear;
                btnFilterClear.Text = model.BtnClear;
                btnFilterStart.Text = model.BtnStart;
                btnFilterStop.Text = model.BtnStop;
                btnExport.Text = model.BtnExport;
                btnFilterClearR.Text = model.BtnClear;

                dgvList.Columns["Column1"].HeaderText = model.ColPhoneNumber;
                dgvList.Columns["Column2"].HeaderText = model.ColStatus;
            }
        }
    }
}
