﻿using Krypton.Toolkit;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Wa.Api;
using WaMarketProApp.Extensions;
using Wa.Model;
using WaMarketProApp.Utils;

namespace WaMarketProApp.ChildFrom
{
    public partial class GroupExtractControl : UserControl
    {
        public override string Text { get; set; } = "Extract";
        public WaApi WaApi { get; set; }
        public GroupExtractControl()
        {
            this.Text = LangUtils.GetLang()?.MenuWaExtract;
            InitializeComponent();
        }

 

        private void btnExport_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.dgvList.Rows.Count == 0)
                {
                    KryptonMessageBox.Show("No Data!", @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Warning, showCtrlCopy: false);
                    return;
                }

                var list=this.dgvList.DataSource as List<GroupMemberItem>;
                var resultList = list.Select(g => g.PhoneNumber).ToArray();
                var dialog = new SaveFileDialog
                {
                    Filter = @"Text File|*.txt",
                    FileName = $"Exist-PhoneList-{DateTime.Now.ToString("yyyy-MM-ddHHmmss")}"
                };
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    var path = dialog.FileName;
                    File.WriteAllLines(path, resultList, Encoding.UTF8);
                    KryptonMessageBox.Show("Save Success!", @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Information, showCtrlCopy: false);

                }
            }
            catch (Exception ex)
            {

                KryptonMessageBox.Show(ex.Message, @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Error, showCtrlCopy: false);
            }
        }

       

        private void btnGet_Click(object sender, EventArgs e)
        {
            
            var gList = this.WaApi.GetGroup();
           
            this.BindGroupList(gList);
        }
        private void BindGroupList(List<GroupModel> gList)
        {
            this.InvokeOnUiThreadIfRequired(() =>
            {
                
                this.comGroup.DataSource = gList;
                this.comGroup.DisplayMember = "GroupName";
                this.comGroup.ValueMember = "GroupId";
                if (gList.Count > 0)
                {
                    this.comGroup.SelectedIndex = 0;
                }

            });
        }

        private void btnGetMember_Click(object sender, EventArgs e)
        {
            if (this.comGroup.Items.Count == 0)
            {
                KryptonMessageBox.Show("Please get your group list!", "Info",
                                MessageBoxButtons.OK, KryptonMessageBoxIcon.Warning, showCtrlCopy: false);
                return;
            }
            var groupId = ((GroupModel)this.comGroup.SelectedItem).GroupId;
            var memberList = this.WaApi.GetGroupMembers(groupId);
            this.InvokeOnUiThreadIfRequired(() =>
            {
                this.dgvList.DataSource = memberList;
                 
            });
        }

        private void btnClearR_Click(object sender, EventArgs e)
        {
            this.dgvList.DataSource = new List<GroupMemberItem>();
        }

        private void GroupExtractControl_Load(object sender, EventArgs e)
        {
            var model = LangUtils.GetLang();
            if (model != null)
            {
                this.Text = model.MenuWaExtract;
                kryptonLabel11.Text = model.LabGroup;
                btnGet.Text = model.BtnGetGroup;
                btnGetMember.Text = model.BtnGetMembers;
                btnExport.Text=model.BtnExport;
                btnClearR.Text = model.BtnClear;
                dgvList.Columns["Column1"].HeaderText = model.ColIndex;
                dgvList.Columns["Column2"].HeaderText = model.ColPhoneNumber;
            }
        }
    }
}
