﻿using Krypton.Toolkit;
using Sqlite.Db.Model;
using Sqlite.Db;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Wa.Api;
using WaMarketProApp.Extensions;
using WaMarketProApp.Model;
using Wa.Model;
using WaMarketProApp.Utils;

namespace WaMarketProApp.ChildFrom
{
    public partial class GroupSendControl : UserControl
    {
        private Random random = new Random();
        private IRepositoryService<ButtonModel> btnService = new RepositoryService<ButtonModel>();
        private IRepositoryService<Sqlite.Db.Model.MessageModel> msgService = new RepositoryService<Sqlite.Db.Model.MessageModel>();
        private CancellationTokenSource cancellationToken = new CancellationTokenSource();
        private List<FilterResultModel> list = new List<FilterResultModel>();
        private int sendGroupSuccess = 0;
        private int sendGroupFailure = 0;
        public override string Text { get; set; } = "GroupSend";
        public WaApi WaApi { get; set; }
        public GroupSendControl()
        {
            this.Text = LangUtils.GetLang()?.MenuWaSend;
            InitializeComponent();
        }

        private void btnImport_Click(object sender, EventArgs e)
        {
            if (!this.WaApi.ExLogin())
            {
                KryptonMessageBox.Show("Please Login your Whatsapp!", "Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Error, showCtrlCopy: false);
                return;
            }
            var gList = this.WaApi.GetGroup();
            this.BindGroupList(gList);
        }
        private void BindGroupList(List<GroupModel> gList)
        {
            this.InvokeOnUiThreadIfRequired(() =>
            {
                this.dgvGroup.DataSource = gList;



            });
        }
        private void btnFilterClear_Click(object sender, EventArgs e)
        {
            this.dgvGroup.DataSource = new List<GroupModel>();
        }

        private async void btnStart_Click(object sender, EventArgs e)
        {
            
            if (!IsChoice())
            {
                KryptonMessageBox.Show("Please Choice group!", "Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Warning, showCtrlCopy: false);
                return;
            }
            var msgList = msgService.GetList(m => m.Enable && m.Type == "Group");
            if (msgList.Count == 0)
            {
                KryptonMessageBox.Show("Please add your message!", "Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Warning, showCtrlCopy: false);
                return;
            }

            var gList = GetChoiceList().ToList();

            this.btnStart.Enabled = false;
            this.btnStop.Enabled = true;

            var token = this.cancellationToken.Token;
            Action<bool, string, string> acResult = (s, name, result) => this.AddSendGroupResult(s, name, result);
            await Task.Factory.StartNew(async () =>
            {
                await this.BulkGroupSend(gList, msgList, acResult, token).ContinueWith(t =>
                {
                    this.cancellationToken = new CancellationTokenSource();
                    this.InvokeOnUiThreadIfRequired(() =>
                    {
                        this.btnStart.Enabled = true;
                        this.btnStop.Enabled = false;
                    });
                });
            }, token);
        }

        private bool IsChoice()
        {
            var list = this.dgvGroup.DataSource as List<GroupModel>;
            return list.Exists(g => g.Choice);
        }

        private List<GroupModel> GetChoiceList()
        {
            var list = this.dgvGroup.DataSource as List<GroupModel>;
            return list.Where(g => g.Choice).ToList();
        }

        private async Task BulkGroupSend(List<GroupModel> groupList, List<Sqlite.Db.Model.MessageModel> msgList, Action<bool, string, string> actionResult, CancellationToken cancellation)
        {
            var sendMsgMin = (int)this.nudSendMsgMin.Value;
            var sendMsgMax = (int)this.nudSendMsgMax.Value;
            var sendMsgNum = (int)this.nudSendMsgNum.Value;
            var sendDelayMin = (int)this.nudSendDelayMin.Value;
            var sendDelayMax = (int)this.nudSendDelayMax.Value;


            var btnMsgList = btnService.GetList();
            var nums = 0;
            foreach (var group in groupList)
            {
                if (cancellation.IsCancellationRequested)
                {
                    break;
                }
                nums++;
                var index = this.random.Next(0, msgList.Count);
                var msgModel = msgList[index];
                var btnMlist = this.btnService.GetList(m => m.MsgId == msgModel.Id)
                    .Select(b => new Buttons { Text = b.Title, Type = b.Type, Url = b.Content }).ToList();
                if (!group.IsAdmin)
                {
                    actionResult(false, group.GroupName, "Only Admins can send messages!");
                }
                else
                {
                    var c = this.WaApi.SendMessage(group.GroupId, msgModel.Content, msgModel.Title, msgModel.File, btnMlist);
                    actionResult(c.Item1, group.GroupName, c.Item2);
                }
                var delay = this.random.Next(sendDelayMin, sendDelayMax) * 1000;
                await Task.Delay(delay);
                if (nums % sendMsgNum == 0)
                {
                    var delay1 = this.random.Next(sendMsgMin, sendMsgMax) * 1000;
                    await Task.Delay(delay1);
                }
            }


        }

        private void AddSendGroupResult(bool status, string groupName, string result)
        {

            var s = status ? "Success" : "Fail";
            this.InvokeOnUiThreadIfRequired(() =>
            {
                dgvList.Rows.Add(new object[] { groupName, s, result });
                if (status)
                {
                    this.sendGroupSuccess++;
                    this.labSuccess.Text = this.sendGroupSuccess.ToString();
                }
                else
                {
                    this.sendGroupFailure++;
                    this.labFail.Text = this.sendGroupFailure.ToString();
                }

            });
        }


        private void btnStop_Click(object sender, EventArgs e)
        {
            this.cancellationToken.Cancel();
        }

        private void btnExport_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.dgvList.Rows.Count == 0)
                {
                    KryptonMessageBox.Show("No Data!", @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Warning, showCtrlCopy: false);
                    return;
                }

                var resultList = list.Select(g => g.PhoneNumber).ToArray();
                var dialog = new SaveFileDialog
                {
                    Filter = @"Text File|*.txt",
                    FileName = $"Exist-PhoneList-{DateTime.Now.ToString("yyyy-MM-ddHHmmss")}"
                };
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    var path = dialog.FileName;
                    File.WriteAllLines(path, resultList, Encoding.UTF8);
                    KryptonMessageBox.Show("Save Success!", @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Information, showCtrlCopy: false);

                }
            }
            catch (Exception ex)
            {

                KryptonMessageBox.Show(ex.Message, @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Error, showCtrlCopy: false);
            }
        }

        private void btnClearR_Click(object sender, EventArgs e)
        {
            this.dgvList.Rows.Clear();
            this.list.Clear();
            this.sendGroupFailure = 0;
            this.sendGroupSuccess = 0;
            this.labSuccess.Text = "0";
            this.labFail.Text = "0";
        }

        private void cbAll_CheckedChanged(object sender, EventArgs e)
        {
            var state = this.cbAll.Checked;
            if (this.dgvGroup.Rows.Count > 0)
            {
                foreach (DataGridViewRow row in this.dgvGroup.Rows)
                {
                    row.Cells["ColChoice"].Value = state;
                }
            }
        }

        private void SetLang()
        {
            var model = LangUtils.GetLang();
            if (model != null)
            {
                this.Text = model.MenuWaSend;
                kryptonGroupBox1.Values.Heading = model.MenuGroupContact;
                kryptonLabel1.Text = model.LabTotal;
                kryptonLabel2.Text = model.LabSucces;
                kryptonLabel3.Text = model.LabFail;
                btnGet.Text = model.BtnGet;
                btnClear.Text = model.BtnClear;
                btnStart.Text = model.BtnStart;
                btnStop.Text = model.BtnStop;
                btnClearR.Text = model.BtnClear;
                kryptonLabel4.Text = model.LabWait;
                kryptonLabel5.Text = model.LabWait;
                kryptonLabel8.Text = model.LabMessage;
                kryptonLabel7.Text = model.LabSeconds;
                kryptonLabel10.Text = model.LabSeconds1;

                dgvList.Columns["Column1"].HeaderText = model.ColPhoneNumber;
                dgvList.Columns["Column2"].HeaderText = model.ColStatus;
                dgvList.Columns["Column3"].HeaderText = model.ColReason;

                dgvGroup.Columns["ColChoice"].HeaderText = model.ColChoice;
                dgvGroup.Columns["Column4"].HeaderText = model.ColGroupName;
            }
        }

        private void GroupSendControl_Load(object sender, EventArgs e)
        {
            SetLang();
        }
    }
}
