﻿using Krypton.Toolkit;
using Sqlite.Db.Model;
using Sqlite.Db;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Wa.Api;
using WaMarketProApp.Extensions;
using WaMarketProApp.Model;
using WaMarketProApp.Utils;

namespace WaMarketProApp.ChildFrom
{
    public partial class JoinGroupControl : UserControl
    {
        private Random random = new Random();
        private IRepositoryService<ButtonModel> btnService = new RepositoryService<ButtonModel>();
        private IRepositoryService<Sqlite.Db.Model.MessageModel> msgService = new RepositoryService<Sqlite.Db.Model.MessageModel>();
        private CancellationTokenSource cancellationToken = new CancellationTokenSource();
        private List<FilterResultModel> list = new List<FilterResultModel>();
        private int fail = 0;
        private int success = 0;
        public override string Text { get; set; } = "Join";
        public WaApi WaApi { get; set; }
        public JoinGroupControl()
        {
            this.Text = LangUtils.GetLang()?.MenuWaJoin;
            InitializeComponent();
        }

        private void btnImport_Click(object sender, EventArgs e)
        {

            try
            {

                var dialog = new OpenFileDialog
                {
                    Filter = @"Text File|*.txt",
                    Title = @"Select Text File"
                };
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    var path = dialog.FileName;
                    var list = File.ReadAllLines(path)
                         .Where(s => !string.IsNullOrEmpty(s)).ToArray();
                    this.txtList.Lines = list;
                    this.labTotal.Text = list.Count().ToString();
                }
            }
            catch (Exception ex)
            {
                KryptonMessageBox.Show(ex.Message, @"Error", MessageBoxButtons.OK, KryptonMessageBoxIcon.Error, showCtrlCopy: false);

            }
        }

        private void btnClear1_Click(object sender, EventArgs e)
        {
            this.txtList.Clear();
            this.labTotal.Text = "0";
        }

        private async void btnStart_Click(object sender, EventArgs e)
        {
            
            if (string.IsNullOrEmpty(this.txtList.Text))
            {
                KryptonMessageBox.Show("Please get your whatsapp contacts!", "Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Warning, showCtrlCopy: false);
                return;
            }
           

            this.btnStart.Enabled = false;
            this.btnStop.Enabled = true;
            var gList = this.txtList.Lines.ToList();
            var token = this.cancellationToken.Token;
            Action<bool, string> acResult = (r, phone) => this.AddResult(r, phone, this.dgvList);
            await Task.Factory.StartNew(async () =>
            {
                await this.BulkJoin(gList, acResult, token).ContinueWith(t =>
                {
                    this.cancellationToken = new CancellationTokenSource();
                    this.InvokeOnUiThreadIfRequired(() =>
                    {
                        this.btnStart.Enabled = true;
                        this.btnStop.Enabled = false;
                    });
                });
            }, token);
        }

        private async Task BulkJoin(List<string> groupList, Action<bool, string> actionResult, CancellationToken cancellation)
        {
            var sendDelayMin = (int)this.nudSendDelayMin.Value;
            var sendDelayMax = (int)this.nudSendDelayMax.Value;
            foreach (var group in groupList)
            {
                if (cancellation.IsCancellationRequested)
                {
                    break;
                }
                var c = this.WaApi.JoinGroup(group);
                actionResult(c, group);
                var delay = this.random.Next(sendDelayMin, sendDelayMax) * 1000;
                await Task.Delay(delay);
                
            }


        }

        private void AddResult(bool status, string group, KryptonDataGridView dgv)
        {
            
            var s = status ? "Success" : "Fail";
            this.InvokeOnUiThreadIfRequired(() =>
            {
                dgv.Rows.Add(new object[] { group, s });
                if (status)
                {
                    this.success++;
                    this.labSuccess.Text = this.success.ToString();
                }
                else
                {
                    this.fail++;
                    this.labFail.Text = this.fail.ToString();
                }

            });
        }


        private void btnStop_Click(object sender, EventArgs e)
        {
            this.cancellationToken.Cancel();
        }

        private void btnExport_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.dgvList.Rows.Count == 0)
                {
                    KryptonMessageBox.Show("No Data!", @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Warning, showCtrlCopy: false);
                    return;
                }

                var resultList = list.Select(g => g.PhoneNumber).ToArray();
                var dialog = new SaveFileDialog
                {
                    Filter = @"Text File|*.txt",
                    FileName = $"Exist-PhoneList-{DateTime.Now.ToString("yyyy-MM-ddHHmmss")}"
                };
                if (dialog.ShowDialog() == DialogResult.OK)
                {
                    var path = dialog.FileName;
                    File.WriteAllLines(path, resultList, Encoding.UTF8);
                    KryptonMessageBox.Show("Save Success!", @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Information, showCtrlCopy: false);

                }
            }
            catch (Exception ex)
            {

                KryptonMessageBox.Show(ex.Message, @"Info", MessageBoxButtons.OK, KryptonMessageBoxIcon.Error, showCtrlCopy: false);
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            this.dgvList.Rows.Clear();
            this.list.Clear();
            this.fail = 0;
            this.success = 0;
            this.labSuccess.Text = "0";
            this.labFail.Text = "0";
        }

        private void JoinGroupControl_Load(object sender, EventArgs e)
        {
            var model = LangUtils.GetLang();
            if (model != null)
            {
                this.Text = model.MenuWaJoin;
                kryptonGroupBox1.Values.Heading = model.GroupBoxGroupLink;
                kryptonLabel1.Text = model.LabTotal;
                kryptonLabel2.Text = model.LabSucces;
                kryptonLabel3.Text = model.LabFail;
                btnImport.Text=model.BtnImport;
                btnClear1.Text = model.BtnClear;
                btnStart.Text=model.BtnStart;
                btnStop.Text=model.BtnStop;
                btnClear.Text = model.BtnClear;
                kryptonLabel5.Text = model.LabWait;
                kryptonLabel10.Text = model.LabSeconds;

                dgvList.Columns["Column1"].HeaderText = model.ColGroupLink;
                dgvList.Columns["Column2"].HeaderText = model.ColStatus;

               
            }
        }
    }
}
