﻿using LinkDotNet.NUniqueHardwareID;
using Newtonsoft.Json;
using System;
using System.IO;
using System.Text;
using WaMarketProApp.Model;

namespace WaMarketProApp.Utils
{
    public class AuthUtils
    {
        private static string path = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "WaMarketProApp");
        private static string fileName = "Key.txt";
        public static bool IsActive()
        {
            var activeCode = GenerateUniqueId();
            var licenseStr = ReadLicenseKey();
            return Verify(activeCode, licenseStr);

        }



        public static bool SaveLicenseKey(string licenseStr)
        {
            var activeCode = GenerateUniqueId();
            if (Verify(activeCode, licenseStr))
            {
                SaveLicense(licenseStr);
                return true;
            }
            return false;

        }


        private static bool Verify(string activeCode, string licenseStr)
        {
            if (string.IsNullOrEmpty(licenseStr))
            {
                return false;
            }
            var deLicenseStr = Base64Decode(licenseStr);
            var keyModel = JsonConvert.DeserializeObject<ActiveModel>(deLicenseStr);
            if (keyModel == null)
            {
                return false;
            }
            var deActiveCode = Base64Decode(keyModel.ActiveCode);
            if (deActiveCode == activeCode || deActiveCode == "Admin")
            {
                if (keyModel.End > DateTime.Now)
                {
                    return true;
                }
            }
            return false;

        }


        private static string ReadLicenseKey()
        {
            try
            {
                var filePath = Path.Combine(path, fileName);
                if (File.Exists(filePath))
                {
                    var str = File.ReadAllText(filePath, Encoding.UTF8);
                    return str;

                }
                return string.Empty;

            }
            catch (Exception ex)
            {
                return string.Empty;
            }

        }

        private static void SaveLicense(string licenseStr)
        {
            var filePath = Path.Combine(path, fileName);
            if (!Directory.Exists(path))
            {
                Directory.CreateDirectory(path);
            }
            File.WriteAllText(filePath, licenseStr, Encoding.UTF8);
        }

        public static string Base64Decode(string str)
        {
            var bytes = Convert.FromBase64String(str);
            return Encoding.UTF8.GetString(bytes);
        }

        public static string GenerateUniqueId()
        {
            var hardwareIdGenerator = new UniqueHardwareId
            {
                UseCPUInformation = true,
                UseMACAddress = false,
                UseVolumeInformation = false,
            };

            return hardwareIdGenerator.CalculateHardwareId();
        }

    }



}
